@@ -28,6 +28,7 @@
 
 using System;
 using System.Collections;
+using System.Collections.Generic;
 using System.IO;
 using System.Xml;
 using System.Xml.Serialization;
@@ -37,8 +38,13 @@ using Mono.Addins.Database;
 
 namespace Mono.Addins.Description
 {
-	// This class represent an add-in configuration file. It has properties for getting
-	// all information, and methods for loading and saving files.
+	/// <summary>
+	/// An add-in description
+	/// </summary>
+	/// <remarks>
+	/// This class represent an add-in manifest. It has properties for getting
+	/// all information, and methods for loading and saving files.
+	///	</remarks>
 	public class AddinDescription: IBinaryXmlElement
 	{
 		XmlDocument configDoc;
@@ -72,6 +78,8 @@ namespace Mono.Addins.Description
 		ExtensionNodeDescription localizer;
 		object[] fileInfo;
 		
+		AddinPropertyCollectionImpl properties;
+		
 		internal static BinaryXmlTypeMap typeMap;
 		
 		static AddinDescription ()
@@ -90,6 +98,7 @@ namespace Mono.Addins.Description
 			typeMap.RegisterType (typeof(AssemblyDependency), "AssemblyDependency");
 			typeMap.RegisterType (typeof(NodeTypeAttribute), "NodeTypeAttribute");
 			typeMap.RegisterType (typeof(AddinFileInfo), "FileInfo");
+			typeMap.RegisterType (typeof(AddinProperty), "Property");
 		}
 		
 		internal AddinDatabase OwnerDatabase {
@@ -97,27 +106,63 @@ namespace Mono.Addins.Description
 			set { ownerDatabase = value; }
 		}
 		
+		/// <summary>
+		/// Gets or sets the path to the main addin file.
+		/// </summary>
+		/// <value>
+		/// The addin file.
+		/// </value>
+		/// <remarks>
+		/// The add-in file can be either the main assembly of an add-in or an xml manifest.
+		/// </remarks>
 		public string AddinFile {
 			get { return sourceAddinFile; }
 			set { sourceAddinFile = value; }
 		}
 		
+		/// <summary>
+		/// Gets the addin identifier.
+		/// </summary>
+		/// <value>
+		/// The addin identifier.
+		/// </value>
 		public string AddinId {
 			get { return Addin.GetFullId (Namespace, LocalId, Version); }
 		}
 		
+		/// <summary>
+		/// Gets or sets the local identifier.
+		/// </summary>
+		/// <value>
+		/// The local identifier.
+		/// </value>
 		public string LocalId {
 			get { return id != null ? id : string.Empty; }
 			set { id = value; hasUserId = true; }
 		}
 
+		/// <summary>
+		/// Gets or sets the namespace.
+		/// </summary>
+		/// <value>
+		/// The namespace.
+		/// </value>
 		public string Namespace {
 			get { return ns != null ? ns : string.Empty; }
 			set { ns = value; }
 		}
 
+		/// <summary>
+		/// Gets or sets the display name of the add-in.
+		/// </summary>
+		/// <value>
+		/// The name.
+		/// </value>
 		public string Name {
 			get {
+				string val = Properties.GetPropertyValue ("Name");
+				if (val.Length > 0)
+					return val;
 				if (name != null && name.Length > 0)
 					return name;
 				if (HasUserId)
@@ -129,42 +174,115 @@ namespace Mono.Addins.Description
 			}
 			set { name = value; }
 		}
-
+		
+		/// <summary>
+		/// Gets or sets the version.
+		/// </summary>
+		/// <value>
+		/// The version.
+		/// </value>
 		public string Version {
 			get { return version != null ? version : string.Empty; }
 			set { version = value; }
 		}
 
+		/// <summary>
+		/// Gets or sets the version of the add-in with which this add-in is backwards compatible.
+		/// </summary>
+		/// <value>
+		/// The compat version.
+		/// </value>
 		public string CompatVersion {
 			get { return compatVersion != null ? compatVersion : string.Empty; }
 			set { compatVersion = value; }
 		}
 
+		/// <summary>
+		/// Gets or sets the author.
+		/// </summary>
+		/// <value>
+		/// The author.
+		/// </value>
 		public string Author {
-			get { return author != null ? author : string.Empty; }
+			get {
+				string val = Properties.GetPropertyValue ("Author");
+				if (val.Length > 0)
+					return val;
+				return author ?? string.Empty; 
+			}
 			set { author = value; }
 		}
 
+		/// <summary>
+		/// Gets or sets the Url where more information about the add-in can be found.
+		/// </summary>
+		/// <value>
+		/// The URL.
+		/// </value>
 		public string Url {
-			get { return url != null ? url : string.Empty; }
+			get {
+				string val = Properties.GetPropertyValue ("Url");
+				if (val.Length > 0)
+					return val;
+				return url ?? string.Empty;
+			}
 			set { url = value; }
 		}
 
+		/// <summary>
+		/// Gets or sets the copyright.
+		/// </summary>
+		/// <value>
+		/// The copyright.
+		/// </value>
 		public string Copyright {
-			get { return copyright != null ? copyright : string.Empty; }
+			get {
+				string val = Properties.GetPropertyValue ("Copyright");
+				if (val.Length > 0)
+					return val;
+				return copyright ?? string.Empty; 
+			}
 			set { copyright = value; }
 		}
 
+		/// <summary>
+		/// Gets or sets the description of the add-in.
+		/// </summary>
+		/// <value>
+		/// The description.
+		/// </value>
 		public string Description {
-			get { return description != null ? description : string.Empty; }
+			get {
+				string val = Properties.GetPropertyValue ("Description");
+				if (val.Length > 0)
+					return val;
+				return description ?? string.Empty;
+			}
 			set { description = value; }
 		}
 
+		/// <summary>
+		/// Gets or sets the category of the add-in.
+		/// </summary>
+		/// <value>
+		/// The category.
+		/// </value>
 		public string Category {
-			get { return category != null ? category : string.Empty; }
+			get {
+				string val = Properties.GetPropertyValue ("Category");
+				if (val.Length > 0)
+					return val;
+				return category ?? string.Empty;
+			}
 			set { category = value; }
 		}
 		
+		/// <summary>
+		/// Gets the base path for locating external files relative to the add-in.
+		/// </summary>
+		/// <value>
+		/// The base path.
+		/// </value>
 		public string BasePath {
 			get { return basePath != null ? basePath : string.Empty; }
 		}
@@ -174,16 +292,34 @@ namespace Mono.Addins.Description
 			basePath = path;
 		}
 		
+		/// <summary>
+		/// Gets or sets a value indicating whether this instance is an add-in root.
+		/// </summary>
+		/// <value>
+		/// <c>true</c> if this instance is an add-in root; otherwise, <c>false</c>.
+		/// </value>
 		public bool IsRoot {
 			get { return isroot; }
 			set { isroot = value; }
 		}
 		
+		/// <summary>
+		/// Gets or sets a value indicating whether this add-in is enabled by default.
+		/// </summary>
+		/// <value>
+		/// <c>true</c> if enabled by default; otherwise, <c>false</c>.
+		/// </value>
 		public bool EnabledByDefault {
 			get { return defaultEnabled; }
 			set { defaultEnabled = value; }
 		}
 		
+		/// <summary>
+		/// Gets or sets the add-in flags.
+		/// </summary>
+		/// <value>
+		/// The flags.
+		/// </value>
 		public AddinFlags Flags {
 			get { return flags; }
 			set { flags = value; }
@@ -194,14 +330,32 @@ namespace Mono.Addins.Description
 			set { hasUserId = value; }
 		}
 		
+		/// <summary>
+		/// Gets a value indicating whether this add-in can be disabled.
+		/// </summary>
+		/// <value>
+		/// <c>true</c> if this add-in can be disabled; otherwise, <c>false</c>.
+		/// </value>
 		public bool CanDisable {
 			get { return (flags & AddinFlags.CantDisable) == 0 && !IsHidden; }
 		}
 		
+		/// <summary>
+		/// Gets a value indicating whether this add-in can be uninstalled.
+		/// </summary>
+		/// <value>
+		/// <c>true</c> if this instance can be uninstalled; otherwise, <c>false</c>.
+		/// </value>
 		public bool CanUninstall {
 			get { return (flags & AddinFlags.CantUninstall) == 0 && !IsHidden; }
 		}
 		
+		/// <summary>
+		/// Gets a value indicating whether this add-in is hidden.
+		/// </summary>
+		/// <value>
+		/// <c>true</c> if this add-in is hidden; otherwise, <c>false</c>.
+		/// </value>
 		public bool IsHidden {
 			get { return (flags & AddinFlags.Hidden) != 0; }
 		}
@@ -212,6 +366,15 @@ namespace Mono.Addins.Description
 				   (CompatVersion.Length == 0 || Addin.CompareVersions (ver, CompatVersion) <= 0);
 		}
 		
+		/// <summary>
+		/// Gets all external files
+		/// </summary>
+		/// <value>
+		/// All files.
+		/// </value>
+		/// <remarks>
+		/// External files are data files and assemblies explicitly referenced in the Runtime section of the add-in manifest.
+		/// </remarks>
 		public StringCollection AllFiles {
 			get {
 				StringCollection col = new StringCollection ();
@@ -226,6 +389,12 @@ namespace Mono.Addins.Description
 			}
 		}
 		
+		/// <summary>
+		/// Gets all paths to be ignored by the add-in scanner.
+		/// </summary>
+		/// <value>
+		/// All paths to be ignored.
+		/// </value>
 		public StringCollection AllIgnorePaths {
 			get {
 				StringCollection col = new StringCollection ();
@@ -240,6 +409,12 @@ namespace Mono.Addins.Description
 			}
 		}
 		
+		/// <summary>
+		/// Gets the main module.
+		/// </summary>
+		/// <value>
+		/// The main module.
+		/// </value>
 		public ModuleDescription MainModule {
 			get {
 				if (mainModule == null) {
@@ -253,6 +428,17 @@ namespace Mono.Addins.Description
 			}
 		}
 		
+		/// <summary>
+		/// Gets the optional modules.
+		/// </summary>
+		/// <value>
+		/// The optional modules.
+		/// </value>
+		/// <remarks>
+		/// Optional modules can be used to declare extensions which will be registered only if some specified
+		/// add-in dependencies can be satisfied. Dependencies specified in optional modules are 'soft dependencies',
+		/// which means that they don't need to be satisfied in order to load the add-in.
+		/// </remarks>
 		public ModuleCollection OptionalModules {
 			get {
 				if (optionalModules == null) {
@@ -266,6 +452,12 @@ namespace Mono.Addins.Description
 			}
 		}
 		
+		/// <summary>
+		/// Gets all modules (including the main module and all optional modules)
+		/// </summary>
+		/// <value>
+		/// All modules.
+		/// </value>
 		public ModuleCollection AllModules {
 			get {
 				ModuleCollection col = new ModuleCollection (this);
@@ -276,6 +468,12 @@ namespace Mono.Addins.Description
 			}
 		}
 		
+		/// <summary>
+		/// Gets the extension node sets.
+		/// </summary>
+		/// <value>
+		/// The extension node sets.
+		/// </value>
 		public ExtensionNodeSetCollection ExtensionNodeSets {
 			get {
 				if (nodeSets == null) {
@@ -289,6 +487,12 @@ namespace Mono.Addins.Description
 			}
 		}
 		
+		/// <summary>
+		/// Gets the extension points.
+		/// </summary>
+		/// <value>
+		/// The extension points.
+		/// </value>
 		public ExtensionPointCollection ExtensionPoints {
 			get {
 				if (extensionPoints == null) {
@@ -302,6 +506,12 @@ namespace Mono.Addins.Description
 			}
 		}
 		
+		/// <summary>
+		/// Gets the condition types.
+		/// </summary>
+		/// <value>
+		/// The condition types.
+		/// </value>
 		public ConditionTypeDescriptionCollection ConditionTypes {
 			get {
 				if (conditionTypes == null) {
@@ -315,11 +525,37 @@ namespace Mono.Addins.Description
 			}
 		}
 		
+		/// <summary>
+		/// Gets or sets the add-in localizer.
+		/// </summary>
+		/// <value>
+		/// The description of the add-in localizer for this add-in.
+		/// </value>
 		public ExtensionNodeDescription Localizer {
 			get { return localizer; }
 			set { localizer = value; }
 		}
 		
+		/// <summary>
+		/// Custom properties specified in the add-in header
+		/// </summary>
+		public AddinPropertyCollection Properties {
+			get {
+				if (properties == null)
+					properties = new AddinPropertyCollectionImpl ();
+				return properties;
+			}
+		}
+		
+		/// <summary>
+		/// Adds an extension point.
+		/// </summary>
+		/// <returns>
+		/// The extension point.
+		/// </returns>
+		/// <param name='path'>
+		/// Path that identifies the new extension point.
+		/// </param>
 		public ExtensionPoint AddExtensionPoint (string path)
 		{
 			ExtensionPoint ep = new ExtensionPoint ();
@@ -376,6 +612,12 @@ namespace Mono.Addins.Description
 			}
 		}
 		
+		/// <summary>
+		/// Gets or sets file where this description is stored
+		/// </summary>
+		/// <value>
+		/// The file path.
+		/// </value>
 		public string FileName {
 			get { return configFile; }
 			set { configFile = value; }
@@ -416,12 +658,64 @@ namespace Mono.Addins.Description
 			return false;
 		}
 		
+		void TransferCoreProperties (bool removeProperties)
+		{
+			if (properties == null)
+				return;
+			
+			string val = properties.ExtractCoreProperty ("Id", removeProperties);
+			if (val != null)
+				id = val;
+			
+			val = properties.ExtractCoreProperty ("Namespace", removeProperties);
+			if (val != null)
+				ns = val;
+			
+			val = properties.ExtractCoreProperty ("Version", removeProperties);
+			if (val != null)
+				version = val;
+			
+			val = properties.ExtractCoreProperty ("CompatVersion", removeProperties);
+			if (val != null)
+				compatVersion = val;
+			
+			val = properties.ExtractCoreProperty ("DefaultEnabled", removeProperties);
+			if (val != null)
+				defaultEnabled = GetBool (val, true);
+			
+			val = properties.ExtractCoreProperty ("IsRoot", removeProperties);
+			if (val != null)
+				isroot = GetBool (val, true);
+			
+			val = properties.ExtractCoreProperty ("Flags", removeProperties);
+			if (val != null)
+				flags = (AddinFlags) Enum.Parse (typeof(AddinFlags), val);
+		}
+		
+		/// <summary>
+		/// Saves the add-in description.
+		/// </summary>
+		/// <param name='fileName'>
+		/// File name where to save this instance
+		/// </param>
+		/// <remarks>
+		/// Saves the add-in description to the specified file and sets the FileName property.
+		/// </remarks>
 		public void Save (string fileName)
 		{
 			configFile = fileName;
 			Save ();
 		}
 		
+		/// <summary>
+		/// Saves the add-in description.
+		/// </summary>
+		/// <exception cref='InvalidOperationException'>
+		/// It is thrown if FileName is not set
+		/// </exception>
+		/// <remarks>
+		/// The description is saved to the file specified in the FileName property.
+		/// </remarks>
 		public void Save ()
 		{
 			if (configFile == null)
@@ -436,6 +730,12 @@ namespace Mono.Addins.Description
 			}
 		}
 		
+		/// <summary>
+		/// Generates an XML representation of the add-in description
+		/// </summary>
+		/// <returns>
+		/// An XML manifest.
+		/// </returns>
 		public XmlDocument SaveToXml ()
 		{
 			SaveXml ();
@@ -456,39 +756,20 @@ namespace Mono.Addins.Description
 			
 			elem = configDoc.DocumentElement;
 			
-			if (HasUserId)
-				elem.SetAttribute ("id", id);
-			else
-				elem.RemoveAttribute ("id");
-			
-			elem.SetAttribute ("version", version);
-			elem.SetAttribute ("namespace", ns);
-			
-			if (isroot)
-				elem.SetAttribute ("isroot", "true");
-			else
-				elem.RemoveAttribute ("isroot");
+			SaveCoreProperty (elem, HasUserId ? id : null, "id", "Id");
+			SaveCoreProperty (elem, version, "version", "Version");
+			SaveCoreProperty (elem, ns, "namespace", "Namespace");
+			SaveCoreProperty (elem, isroot ? "true" : null, "isroot", "IsRoot");
 			
 			// Name will return the file name when HasUserId=false
-			if (Name.Length > 0)
-				elem.SetAttribute ("name", Name);
+			if (!string.IsNullOrEmpty (name))
+				elem.SetAttribute ("name", name);
 			else
 				elem.RemoveAttribute ("name");
 				
-			if (compatVersion != null && compatVersion.Length > 0)
-				elem.SetAttribute ("compatVersion", compatVersion);
-			else
-				elem.RemoveAttribute ("compatVersion");
-			
-			if (defaultEnabled)
-				elem.RemoveAttribute ("defaultEnabled");
-			else
-				elem.SetAttribute ("defaultEnabled", "false");
-			
-			if (flags != AddinFlags.None)
-				elem.RemoveAttribute ("flags");
-			else
-				elem.SetAttribute ("flags", flags.ToString ());
+			SaveCoreProperty (elem, compatVersion, "compatVersion", "CompatVersion");
+			SaveCoreProperty (elem, defaultEnabled ? null : "false", "defaultEnabled", "DefaultEnabled");
+			SaveCoreProperty (elem, flags != AddinFlags.None ? flags.ToString () : null, "flags", "Flags");
 			
 			if (author != null && author.Length > 0)
 				elem.SetAttribute ("author", author);
@@ -537,9 +818,53 @@ namespace Mono.Addins.Description
 				
 			if (extensionPoints != null)
 				extensionPoints.SaveXml (elem);
+			
+			XmlElement oldHeader = (XmlElement) elem.SelectSingleNode ("Header");
+			if (properties == null || properties.Count == 0) {
+				if (oldHeader != null)
+					elem.RemoveChild (oldHeader);
+			} else {
+				if (oldHeader == null) {
+					oldHeader = elem.OwnerDocument.CreateElement ("Header");
+					if (elem.FirstChild != null)
+						elem.InsertBefore (oldHeader, elem.FirstChild);
+					else
+						elem.AppendChild (oldHeader);
+				}
+				else
+					oldHeader.RemoveAll ();
+				foreach (var prop in properties) {
+					XmlElement propElem = elem.OwnerDocument.CreateElement (prop.Name);
+					if (!string.IsNullOrEmpty (prop.Locale))
+						propElem.SetAttribute ("locale", prop.Locale);
+					propElem.InnerText = prop.Value ?? string.Empty;
+					oldHeader.AppendChild (propElem);
+				}
+			}
+		}
+		
+		void SaveCoreProperty (XmlElement elem, string val, string attr, string prop)
+		{
+			if (properties != null && properties.HasProperty (prop)) {
+				elem.RemoveAttribute (attr);
+				if (!string.IsNullOrEmpty (val))
+					properties.SetPropertyValue (prop, val);
+				else
+					properties.RemoveProperty (prop);
+			}
+			else if (string.IsNullOrEmpty (val))
+				elem.RemoveAttribute (attr);
+			else
+				elem.SetAttribute (attr, val);
 		}
 		
 
+		/// <summary>
+		/// Load an add-in description from a file
+		/// </summary>
+		/// <param name='configFile'>
+		/// The file.
+		/// </param>
 		public static AddinDescription Read (string configFile)
 		{
 			AddinDescription config;
@@ -550,11 +875,29 @@ namespace Mono.Addins.Description
 			return config;
 		}
 		
+		/// <summary>
+		/// Load an add-in description from a stream
+		/// </summary>
+		/// <param name='stream'>
+		/// The stream
+		/// </param>
+		/// <param name='basePath'>
+		/// The path to be used to resolve relative file paths.
+		/// </param>
 		public static AddinDescription Read (Stream stream, string basePath)
 		{
 			return Read (new StreamReader (stream), basePath);
 		}
 		
+		/// <summary>
+		/// Load an add-in description from a text reader
+		/// </summary>
+		/// <param name='reader'>
+		/// The text reader
+		/// </param>
+		/// <param name='basePath'>
+		/// The path to be used to resolve relative file paths.
+		/// </param>
 		public static AddinDescription Read (TextReader reader, string basePath)
 		{
 			AddinDescription config = new AddinDescription ();
@@ -599,6 +942,18 @@ namespace Mono.Addins.Description
 			if (localizerElem != null)
 				config.localizer = new ExtensionNodeDescription (localizerElem);
 			
+			XmlElement headerElem = (XmlElement) elem.SelectSingleNode ("Header");
+			if (headerElem != null) {
+				foreach (XmlNode node in headerElem.ChildNodes) {
+					XmlElement prop = node as XmlElement;
+					if (prop == null)
+						continue;
+					config.Properties.SetPropertyValue (prop.LocalName, prop.InnerText, prop.GetAttribute ("locale"));
+				}
+			}
+			
+			config.TransferCoreProperties (false);
+			
 			if (config.id.Length > 0)
 				config.hasUserId = true;
 			
@@ -637,7 +992,19 @@ namespace Mono.Addins.Description
 //			BinaryXmlReader.DumpFile (configFile);
 		}
 		
+		/// <summary>
+		/// Verify this instance.
+		/// </summary>
+		/// <remarks>
+		/// This method checks all the definitions in the description and returns a list of errors.
+		/// If the returned list is empty, it means that the description is valid.
+		/// </remarks>
 		public StringCollection Verify ()
+		{
+			return Verify (new AddinFileSystemExtension ());
+		}
+		
+		internal StringCollection Verify (AddinFileSystemExtension fs)
 		{
 			StringCollection errors = new StringCollection ();
 			
@@ -673,7 +1040,7 @@ namespace Mono.Addins.Description
 			if (bp != null) {
 				foreach (string file in AllFiles) {
 					string asmFile = Path.Combine (bp, file);
-					if (!File.Exists (asmFile))
+					if (!fs.FileExists (asmFile))
 						errors.Add ("The file '" + asmFile + "' referenced in the manifest could not be found.");
 				}
 			}
@@ -682,6 +1049,16 @@ namespace Mono.Addins.Description
 				errors.Add ("The attribute 'type' in the Location element is required.");
 			}
 			
+			// Ensure that there are no duplicated properties
+			
+			if (properties != null) {
+				HashSet<string> props = new HashSet<string> ();
+				foreach (var prop in properties) {
+					if (!props.Add (prop.Name + " " + prop.Locale))
+						errors.Add (string.Format ("Property {0} specified more than once", prop.Name + (prop.Locale != null ? " (" + prop.Locale + ")" : "")));
+				}
+			}
+			
 			return errors;
 		}
 		
@@ -730,6 +1107,7 @@ namespace Mono.Addins.Description
 				AddinDescription tmp = desc1;
 				desc1 = desc2; desc2 = tmp;
 			}
+			((AddinPropertyCollectionImpl)desc1.Properties).AddRange (desc2.Properties);
 			desc1.ExtensionPoints.AddRange (desc2.ExtensionPoints);
 			desc1.ExtensionNodeSets.AddRange (desc2.ExtensionNodeSets);
 			desc1.ConditionTypes.AddRange (desc2.ConditionTypes);
@@ -745,6 +1123,7 @@ namespace Mono.Addins.Description
 		
 		void IBinaryXmlElement.Write (BinaryXmlWriter writer)
 		{
+			TransferCoreProperties (true);
 			writer.WriteValue ("id", id);
 			writer.WriteValue ("ns", ns);
 			writer.WriteValue ("isroot", isroot);
@@ -769,6 +1148,7 @@ namespace Mono.Addins.Description
 			writer.WriteValue ("FilesInfo", fileInfo);
 			writer.WriteValue ("Localizer", localizer);
 			writer.WriteValue ("flags", (int)flags);
+			writer.WriteValue ("Properties", properties);
 		}
 		
 		void IBinaryXmlElement.Read (BinaryXmlReader reader)
@@ -797,6 +1177,7 @@ namespace Mono.Addins.Description
 			fileInfo = (object[]) reader.ReadValue ("FilesInfo", null);
 			localizer = (ExtensionNodeDescription) reader.ReadValue ("Localizer");
 			flags = (AddinFlags) reader.ReadInt32Value ("flags");
+			properties = (AddinPropertyCollectionImpl) reader.ReadValue ("Properties", new AddinPropertyCollectionImpl ());
 			
 			if (mainModule != null)
 				mainModule.SetParent (this);
