@@ -28,6 +28,7 @@
 
 
 using System;
+using System.Linq;
 using System.Xml;
 using System.Collections;
 using System.Reflection;
@@ -35,9 +36,21 @@ using System.Reflection;
 using Mono.Addins.Description;
 using Mono.Addins.Database;
 using Mono.Addins.Localization;
+using System.Collections.Generic;
 
 namespace Mono.Addins
 {
+	/// <summary>
+	/// An add-in engine.
+	/// </summary>
+	/// <remarks>
+	/// This class allows hosting several independent add-in engines in a single application domain.
+	/// In general, applications use the AddinManager class to query and manage extensions. This class is static,
+	/// so the API is easily accessible. However, some kind applications may need to use several isolated
+	/// add-in engines, and in this case the AddinManager class can't be used, because it is bound to a single
+	/// add-in engine. Those applications can instead create several instances of the AddinEngine class. Each
+	/// add-in engine can be independently initialized with different add-in registries and extension models.
+	/// </remarks>
 	public class AddinEngine: ExtensionContext
 	{
 		bool initialized;
@@ -47,20 +60,44 @@ namespace Mono.Addins
 		
 		bool checkAssemblyLoadConflicts;
 		Hashtable loadedAddins = new Hashtable ();
-		Hashtable nodeSets = new Hashtable ();
+		Dictionary<string,ExtensionNodeSet> nodeSets = new Dictionary<string, ExtensionNodeSet> ();
 		Hashtable autoExtensionTypes = new Hashtable ();
 		Hashtable loadedAssemblies = new Hashtable ();
 		AddinLocalizer defaultLocalizer;
 		IProgressStatus defaultProgressStatus = new ConsoleProgressStatus (false);
 		
+		/// <summary>
+		/// Raised when there is an error while loading an add-in
+		/// </summary>
 		public static event AddinErrorEventHandler AddinLoadError;
+		
+		/// <summary>
+		/// Raised when an add-in is loaded
+		/// </summary>
 		public static event AddinEventHandler AddinLoaded;
+		
+		/// <summary>
+		/// Raised when an add-in is unloaded
+		/// </summary>
 		public static event AddinEventHandler AddinUnloaded;
 		
+		/// <summary>
+		/// Initializes a new instance of the <see cref="Mono.Addins.AddinEngine"/> class.
+		/// </summary>
 		public AddinEngine ()
 		{
 		}
 		
+		/// <summary>
+		/// Initializes the add-in engine
+		/// </summary>
+		/// <param name="configDir">
+		/// Location of the add-in registry.
+		/// </param>
+		/// <remarks>The add-in engine needs to be initialized before doing any add-in operation.
+		/// When initialized with this method, it will look for add-in in the add-in registry
+		/// located in the specified path.
+		/// </remarks>
 		public void Initialize (string configDir)
 		{
 			if (initialized)
@@ -68,10 +105,79 @@ namespace Mono.Addins
 			
 			Assembly asm = Assembly.GetEntryAssembly ();
 			if (asm == null) asm = Assembly.GetCallingAssembly ();
-			Initialize (configDir, asm);
+			Initialize (asm, configDir, null, null);
 		}
 		
-		internal void Initialize (string configDir, Assembly startupAsm)
+		/// <summary>
+		/// Initializes the add-in engine.
+		/// </summary>
+		/// <param name='configDir'>
+		/// Location of the add-in registry.
+		/// </param>
+		/// <param name='addinsDir'>
+		/// Add-ins directory. If the path is relative, it is considered to be relative
+		/// to the configDir directory.
+		/// </param>
+		/// <remarks>
+		/// The add-in engine needs to be initialized before doing any add-in operation.
+		/// Configuration information about the add-in registry will be stored in the
+		/// provided location. The add-in engine will look for add-ins in the provided
+		/// 'addinsDir' directory.
+		/// 
+		/// When specifying a path, it is possible to use a special folder name as root.
+		/// For example: [Personal]/.config/MyApp. In this case, [Personal] will be replaced
+		/// by the location of the Environment.SpecialFolder.Personal folder. Any value
+		/// of the Environment.SpecialFolder enumeration can be used (always between square
+		/// brackets)
+		/// </remarks>
+		public void Initialize (string configDir, string addinsDir)
+		{
+			if (initialized)
+				return;
+			
+			Assembly asm = Assembly.GetEntryAssembly ();
+			if (asm == null) asm = Assembly.GetCallingAssembly ();
+			Initialize (asm, configDir, addinsDir, null);
+		}
+		
+		/// <summary>
+		/// Initializes the add-in engine.
+		/// </summary>
+		/// <param name='configDir'>
+		/// Location of the add-in registry.
+		/// </param>
+		/// <param name='addinsDir'>
+		/// Add-ins directory. If the path is relative, it is considered to be relative
+		/// to the configDir directory.
+		/// </param>
+		/// <param name='databaseDir'>
+		/// Location of the add-in database. If the path is relative, it is considered to be relative
+		/// to the configDir directory.
+		/// </param>
+		/// <remarks>
+		/// The add-in engine needs to be initialized before doing any add-in operation.
+		/// Configuration information about the add-in registry will be stored in the
+		/// provided location. The add-in engine will look for add-ins in the provided
+		/// 'addinsDir' directory. Cached information about add-ins will be stored in
+		/// the 'databaseDir' directory.
+		/// 
+		/// When specifying a path, it is possible to use a special folder name as root.
+		/// For example: [Personal]/.config/MyApp. In this case, [Personal] will be replaced
+		/// by the location of the Environment.SpecialFolder.Personal folder. Any value
+		/// of the Environment.SpecialFolder enumeration can be used (always between square
+		/// brackets)
+		/// </remarks>
+		public void Initialize (string configDir, string addinsDir, string databaseDir)
+		{
+			if (initialized)
+				return;
+			
+			Assembly asm = Assembly.GetEntryAssembly ();
+			if (asm == null) asm = Assembly.GetCallingAssembly ();
+			Initialize (asm, configDir, addinsDir, databaseDir);
+		}
+		
+		internal void Initialize (Assembly startupAsm, string configDir, string addinsDir, string databaseDir)
 		{
 			if (initialized)
 				return;
@@ -88,7 +194,7 @@ namespace Mono.Addins
 			if (configDir == null || configDir.Length == 0)
 				registry = AddinRegistry.GetGlobalRegistry (this, startupDirectory);
 			else
-				registry = new AddinRegistry (this, configDir, startupDirectory);
+				registry = new AddinRegistry (this, configDir, startupDirectory, addinsDir, databaseDir);
 
 			if (registry.CreateHostAddinsFile (asmFile) || registry.UnknownDomain)
 				registry.Update (new ConsoleProgressStatus (false));
@@ -100,6 +206,9 @@ namespace Mono.Addins
 			AppDomain.CurrentDomain.AssemblyLoad += new AssemblyLoadEventHandler (OnAssemblyLoaded);
 		}
 		
+		/// <summary>
+		/// Finalizes the add-in engine.
+		/// </summary>
 		public void Shutdown ()
 		{
 			initialized = false;
@@ -109,9 +218,15 @@ namespace Mono.Addins
 			registry.Dispose ();
 			registry = null;
 			startupDirectory = null;
-			Clear ();
+			ClearContext ();
 		}
 		
+		/// <summary>
+		/// Sets the default localizer to be used for this add-in engine
+		/// </summary>
+		/// <param name="localizer">
+		/// The add-in localizer
+		/// </param>
 		public void InitializeDefaultLocalizer (IAddinLocalizer localizer)
 		{
 			CheckInitialized ();
@@ -125,15 +240,28 @@ namespace Mono.Addins
 			get { return startupDirectory; }
 		}
 		
+		/// <summary>
+		/// Gets whether the add-in engine has been initialized.
+		/// </summary>
 		public bool IsInitialized {
 			get { return initialized; }
 		}
 		
+		/// <summary>
+		/// Gets the default add-in installer
+		/// </summary>
+		/// <remarks>
+		/// The default installer is used by the CheckInstalled method to request
+		/// the installation of missing add-ins.
+		/// </remarks>
 		public IAddinInstaller DefaultInstaller {
 			get { return installer; }
 			set { installer = value; }
 		}
 		
+		/// <summary>
+		/// Gets the default localizer for this add-in engine
+		/// </summary>
 		public AddinLocalizer DefaultLocalizer {
 			get {
 				CheckInitialized ();
@@ -148,6 +276,9 @@ namespace Mono.Addins
 			get { return this; }
 		}
 		
+		/// <summary>
+		/// Gets the localizer for the add-in that is invoking this property
+		/// </summary>
 		public AddinLocalizer CurrentLocalizer {
 			get {
 				CheckInitialized ();
@@ -160,6 +291,9 @@ namespace Mono.Addins
 			}
 		}
 		
+		/// <summary>
+		/// Gets a reference to the RuntimeAddin object for the add-in that is invoking this property
+		/// </summary>
 		public RuntimeAddin CurrentAddin {
 			get {
 				CheckInitialized ();
@@ -168,6 +302,9 @@ namespace Mono.Addins
 			}
 		}
 		
+		/// <summary>
+		/// Gets the add-in registry bound to this add-in engine
+		/// </summary>
 		public AddinRegistry Registry {
 			get {
 				CheckInitialized ();
@@ -180,11 +317,23 @@ namespace Mono.Addins
 			return (RuntimeAddin) loadedAssemblies [asm];
 		}
 		
-		// This method checks if the specified add-ins are installed.
-		// If some of the add-ins are not installed, it will use
-		// the installer assigned to the DefaultAddinInstaller property
-		// to install them. If the installation fails, or if DefaultAddinInstaller
-		// is not set, an exception will be thrown.
+		/// <summary>
+		/// Checks if the provided add-ins are installed, and requests the installation of those
+		/// which aren't.
+		/// </summary>
+		/// <param name="message">
+		/// Message to show to the user when new add-ins have to be installed.
+		/// </param>
+		/// <param name="addinIds">
+		/// List of IDs of the add-ins to be checked.
+		/// </param>
+		/// <remarks>
+		/// This method checks if the specified add-ins are installed.
+		/// If some of the add-ins are not installed, it will use
+		/// the installer assigned to the DefaultAddinInstaller property
+		/// to install them. If the installation fails, or if DefaultAddinInstaller
+		/// is not set, an exception will be thrown.
+		/// </remarks>
 		public void CheckInstalled (string message, params string[] addinIds)
 		{
 			ArrayList notInstalled = new ArrayList ();
@@ -214,7 +363,16 @@ namespace Mono.Addins
 			get { return checkAssemblyLoadConflicts; }
 			set { checkAssemblyLoadConflicts = value; }
 		}
-
+		
+		/// <summary>
+		/// Checks if an add-in has been loaded.
+		/// </summary>
+		/// <param name="id">
+		/// Full identifier of the add-in.
+		/// </param>
+		/// <returns>
+		/// True if the add-in is loaded.
+		/// </returns>
 		public bool IsAddinLoaded (string id)
 		{
 			CheckInitialized ();
@@ -247,6 +405,21 @@ namespace Mono.Addins
 			}
 		}
 		
+		/// <summary>
+		/// Forces the loading of an add-in.
+		/// </summary>
+		/// <param name="statusMonitor">
+		/// Status monitor to keep track of the loading process.
+		/// </param>
+		/// <param name="id">
+		/// Full identifier of the add-in to load.
+		/// </param>
+		/// <remarks>
+		/// This method loads all assemblies that belong to an add-in in memory.
+		/// All add-ins on which the specified add-in depends will also be loaded.
+		/// Notice that in general add-ins don't need to be explicitely loaded using
+		/// this method, since the add-in engine will load them on demand.
+		/// </remarks>
 		public void LoadAddin (IProgressStatus statusMonitor, string id)
 		{
 			CheckInitialized ();
@@ -324,7 +497,7 @@ namespace Mono.Addins
 					// Load the extension points and other addin data
 					
 					foreach (ExtensionNodeSet rel in description.ExtensionNodeSets) {
-						RegisterNodeSet (rel);
+						RegisterNodeSet (iad.Id, rel);
 					}
 					
 					foreach (ConditionTypeDescription cond in description.ConditionTypes) {
@@ -421,14 +594,16 @@ namespace Mono.Addins
 			return true;
 		}
 		
-		internal void RegisterNodeSet (ExtensionNodeSet nset)
+		internal void RegisterNodeSet (string addinId, ExtensionNodeSet nset)
 		{
+			nset.SourceAddinId = addinId;
 			nodeSets [nset.Id] = nset;
 		}
 		
-		internal void UnregisterNodeSet (ExtensionNodeSet nset)
+		internal void UnregisterAddinNodeSets (string addinId)
 		{
-			nodeSets.Remove (nset.Id);
+			foreach (var nset in nodeSets.Values.Where (n => n.SourceAddinId == addinId).ToArray ())
+				nodeSets.Remove (nset.Id);
 		}
 		
 		internal string GetNodeTypeAddin (ExtensionNodeSet nset, string type, string callingAddinId)
@@ -451,8 +626,8 @@ namespace Mono.Addins
 			}
 			
 			foreach (string ns in nset.NodeSets) {
-				ExtensionNodeSet regSet = (ExtensionNodeSet) nodeSets [ns];
-				if (regSet == null) {
+				ExtensionNodeSet regSet;
+				if (!nodeSets.TryGetValue (ns, out regSet)) {
 					ReportError ("Unknown node set: " + ns, callingAddinId, null, false);
 					return null;
 				}
@@ -494,7 +669,16 @@ namespace Mono.Addins
 		{
 			if (AddinDatabase.RunningSetupProcess || asm is System.Reflection.Emit.AssemblyBuilder)
 				return;
-			string asmFile = new Uri (asm.CodeBase).LocalPath;
+			string codeBase;
+			try {
+				codeBase = asm.CodeBase;
+			} catch {
+				return;
+			}
+			Uri u;
+			if (!Uri.TryCreate (codeBase, UriKind.Absolute, out u))
+				return;
+			string asmFile = u.LocalPath;
 			Addin ainfo = Registry.GetAddinForHostAssembly (asmFile);
 			if (ainfo != null && !IsAddinLoaded (ainfo.Id)) {
 				AddinDescription adesc = null;
@@ -516,6 +700,15 @@ namespace Mono.Addins
 			}
 		}
 		
+		/// <summary>
+		/// Creates a new extension context.
+		/// </summary>
+		/// <returns>
+		/// The new extension context.
+		/// </returns>
+		/// <remarks>
+		/// Extension contexts can be used to query the extension model using particular condition values.
+		/// </remarks>
 		public ExtensionContext CreateExtensionContext ()
 		{
 			CheckInitialized ();
